@@ -79,7 +79,7 @@ class NormalizedL2Loss(nn.Module):
         super().__init__()
 
     def forward(self, predict: torch.Tensor, target: torch.Tensor):
-        return (functional.normalize(predict) - functional.normalize(target)).pow(2).mean()
+        return (functional.normalize(predict) - functional.normalize(target)).pow(2).sum(1).mean() / 2
 
     @classmethod
     def factory(cls, config: dict = None):
