@@ -24,7 +24,7 @@ class InvertedResidual(nn.Module):
                  se: bool = False, no_linear=InplaceReLU, norm_layer=nn.BatchNorm2d):
         super().__init__()
 
-        se_layer = functools.partial(SELayer, reduction=4, no_linear=HSigmoid) if se else None  # h-sigmoid in figure 4
+        se_layer = functools.partial(SELayer, reduction=4, sigmoid=HSigmoid) if se else None  # h-sigmoid in figure 4
         self.blocks = nn.Sequential(
             # pixel wise
             conv_bn_nl(in_channels, exp_channels, kernel=1, stride=1, norm_layer=norm_layer, no_linear=no_linear),
@@ -48,7 +48,25 @@ def make_divisible(x: float, divisible_by: int = 8):
     return int(round(x / divisible_by) * divisible_by)
 
 
-def build_blocks(in_channels: int, blocks_setting, width_multiple: float, norm_layer=nn.BatchNorm2d):
+def build_blocks(in_channels: int, stride_list: list, width_multiple: float, norm_layer=nn.BatchNorm2d):
+    blocks_setting = [
+        # kernel, exp, out, se, no linear, stride
+        [3, 16, 16, False, InplaceReLU, 1],
+        [3, 64, 24, False, InplaceReLU, stride_list[1]],
+        [3, 72, 24, False, InplaceReLU, 1],
+        [5, 72, 40, True, InplaceReLU, stride_list[2]],
+        [5, 120, 40, True, InplaceReLU, 1],
+        [5, 120, 40, True, InplaceReLU, 1],
+        [3, 240, 80, False, HSwish, stride_list[3]],
+        [3, 200, 80, False, HSwish, 1],
+        [3, 184, 80, False, HSwish, 1],
+        [3, 184, 80, False, HSwish, 1],
+        [3, 480, 112, True, HSwish, 1],
+        [3, 672, 112, True, HSwish, 1],
+        [5, 672, 112, True, HSwish, 1],
+        [5, 672, 160, True, HSwish, stride_list[4]],
+        [5, 960, 160, True, HSwish, 1],
+    ]
     blocks = []
 
     for kernel, exp, out, se, no_linear, stride in blocks_setting:
@@ -64,44 +82,25 @@ def build_blocks(in_channels: int, blocks_setting, width_multiple: float, norm_l
 
 @box.register(tag='model')
 class MobileNetV3(nn.Module):
-    def __init__(self, feature_dim: int = 1000, width_multiple: float = 1.0, dropout_ratio: float = 0.0):
+    def __init__(self, config: MobileNetV3Config):
         super().__init__()
 
-        blocks_setting = [
-            # kernel, exp, out, se, no linear, stride
-            [3, 16, 16, False, InplaceReLU, 1],
-            [3, 64, 24, False, InplaceReLU, 2],
-            [3, 72, 24, False, InplaceReLU, 1],
-            [5, 72, 40, True, InplaceReLU, 2],
-            [5, 120, 40, True, InplaceReLU, 1],
-            [5, 120, 40, True, InplaceReLU, 1],
-            [3, 240, 80, False, HSwish, 2],
-            [3, 200, 80, False, HSwish, 1],
-            [3, 184, 80, False, HSwish, 1],
-            [3, 184, 80, False, HSwish, 1],
-            [3, 480, 112, True, HSwish, 1],
-            [3, 672, 112, True, HSwish, 1],
-            [5, 672, 112, True, HSwish, 1],
-            [5, 672, 160, True, HSwish, 2],
-            [5, 960, 160, True, HSwish, 1],
-        ]
-
         in_channels = 16
         last_channels = 1280
         norm_layer = load_default_batch_norm_2d()
 
-        first_block = conv_bn_nl(in_channels=3, out_channels=in_channels, kernel=3, stride=2,
+        first_block = conv_bn_nl(in_channels=3, out_channels=in_channels, kernel=3, stride=config.stride_list[0],
                                  norm_layer=norm_layer, no_linear=HSwish)
-        blocks, out_channels = build_blocks(in_channels, blocks_setting, width_multiple, norm_layer)
+        blocks, out_channels = build_blocks(in_channels, config.stride_list, config.width_multiple, norm_layer)
 
-        last_conv = make_divisible(960 * width_multiple)
+        last_conv = make_divisible(960 * config.width_multiple)
         last_block = nn.Sequential(
             conv_bn_nl(out_channels, last_conv, kernel=1, stride=1, no_linear=HSwish, norm_layer=norm_layer),
             GlobalPooling(),
             Squeeze(),
             nn.Linear(last_conv, last_channels),
             HSwish(),
-            Classifier(last_channels, feature_dim, dropout=dropout_ratio)
+            Classifier(last_channels, config.feature_dim, dropout=config.dropout_ratio)
         )
 
         self.blocks = nn.Sequential(first_block, *blocks, last_block)
@@ -113,8 +112,7 @@ class MobileNetV3(nn.Module):
     @classmethod
     def factory(cls, config: dict = None):
         jsonschema.validate(config or {}, cls.schema)
-        config = MobileNetV3Config(values=config)
-        return cls(feature_dim=config.feature_dim, width_multiple=config.width_multiple)
+        return cls(config=MobileNetV3Config(values=config))
 
     with open(str(Path(__file__).parent / 'schema' / 'mobilenet_v3_config.json')) as f:
         schema = json.load(f)
